#!/usr/bin/env python3
import sys, re, csv, decimal
import time
import argparse
from lib.utils import *
from lib.paraver import *
from lib.events import *
from lib.genericCSVtoPRV import *
from lib.asm2prv import *
from lib.ParaverLine import *
import lib.ParaverLine
from lib.DisasmRV import *


def offset_parser(offsets_string, paraver_events, paraver_line, last_num_offsets):
    counter = 0

    offsets_strip = offsets_string.strip()
    events = paraver_events.derivedEvents

    if offsets_strip:
        offsets = offsets_string.split(',')
        for offset in offsets:
            value = int(offset, 0)
            paraver_line.addEvent(events[counter], value)
            counter = counter + 1

    aux_counter = counter
    while aux_counter <= last_num_offsets:
        paraver_line.addEvent(events[aux_counter], 0)
        aux_counter = aux_counter + 1

    return counter-1


def getThreadsFromTrace(csvfile):
    data = csv.reader(csvfile, skipinitialspace=True, delimiter=',')

    line = next(data)
    thread_id = dict()

    if line[-1] != 'core-id':
        thread_id["0"] = 1;
        return thread_id

    accum = 0
    for row in data:
        tid = row[-2]
        if tid not in thread_id:
             accum = accum + 1
             thread_id[tid] = accum

    return thread_id


def intToPRV(string, event, paraver_line, last_state):
    value = 0

    if string != "":
        value = int(string, 0)

    if last_state != 0 or value != 0:
        paraver_line.addEvent(event, value)

    return value


def threadID(string, event, paraver_line, last_state):
    paraver_line.threadID = last_state[string] + 1
    return last_state


def coreID(string, event, paraver_line, last_state):
    paraver_line.cpuId = int(string) + 1
    return  last_state


def interfaceDisasm(disasm, paraver_events, paraver_line, last_state):
    last_state.disasmToPRV(disasm, paraver_events.derivedEvents, paraver_line)
    return last_state


def vehaveTraceToPrv(csvfile, prvfile, PrvEvents, threads):

    nThreads = len(threads)
    header = paraverHeader(csvfile, nthreads=nThreads, delimiter=",")
    writePRVFile(prvfile, header)

    csvfile.seek(0)
    data = csv.reader(csvfile, skipinitialspace=True, delimiter=',')

    paraver_line = ParaverLine(2, 1, 1, 1, 1)

    last_dst_value = 0
    last_dst_event = PrvEvents[2].derivedEvents[0]

    disasm_col = 2
    offset_col = 8
    threadid_col = 9
    coreid_col = 10

    last_state = [0] * 11
    parser_functions = [intToPRV] * 11

    disasm_parser = DisasmRV()
    last_state[disasm_col] = disasm_parser
    parser_functions[disasm_col] = interfaceDisasm

    last_state[offset_col] = -1
    parser_functions[offset_col] = offset_parser

    parser_functions[coreid_col] = coreID

    last_state[threadid_col] = threads
    parser_functions[threadid_col] = threadID

    for row in data:

        if (sanityCheck(row, True) == -1): continue

        paraver_line.time = row[0]
        paraver_line.events = []

        i = 0

        for col in row[1:]:
            i = i + 1
            last_state[i] = parser_functions[i](col, PrvEvents[i], paraver_line, last_state[i])

        writePRVFile(prvfile, paraver_line.getLine())

    printMsg("PRV generated successfully!", "OK")


def generateOffsetEvents(num_events, start_ID):
    offset_event = PrvEvent("addres-VL", start_ID)

    counter = 0

    for i in range(start_ID + 1, start_ID + num_events + 1):
        name = "addres-VL-" + str(counter)
        tmp_event = PrvEvent(name, i)
        counter = counter + 1
        offset_event.derivedEvents.append(tmp_event)

    return offset_event

def parseArguments():

    parser = argparse.ArgumentParser()

    parser.add_argument("input", help="Vehave trace file")
    parser.add_argument("--output-dir", default=".", help="Path to output directory. If not specified will be cwd")
    parser.add_argument("--output-name", help="Paraver trace name. If not specified will have the same basename than the input")
    parser.add_argument("--events-sorted", action='store_true', help="Paraver events sorted")
    args = parser.parse_args()

    ParaverLine.GET_LINE_SORTED = args.events_sorted

    return args

def main():
    args = parseArguments()
    filename = ntpath.basename(args.input)

    if args.output_name is None:
        args.output_name=filename

    start = time.time()
    csvfile = openCSVFile(args.input)
    prvfile = openPRVFile(args.output_name, args.output_dir, compress=False)

    eventsNames = readGenericCSVHeader(csvfile, ",")

    prvEvents = getStdEventsFromJsonFile(eventsNames, "vehave")

    scriptpath = os.path.dirname(os.path.realpath(__file__))

    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[0])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[1])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[2])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[3])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[10])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[12])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Vtype", prvEvents[2].derivedEvents[6])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Vtype", prvEvents[2].derivedEvents[7])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Instructions", prvEvents[2].derivedEvents[9])

    prvEvents[8] = generateOffsetEvents(2048, 48000000)

    thread_ids = getThreadsFromTrace(csvfile)

    nthreads = len(thread_ids)
    printMsg("Number of threads in trace is {}".format(nthreads), "INFO")

    end = time.time()

    pre_time = round(end - start, 2)
    printMsg("Preprocessing time: " + str(pre_time) + " seconds", "INFO")

    start = time.time()

    vehaveTraceToPrv(csvfile, prvfile, prvEvents, thread_ids)
    createRow(args.output_name, args.output_dir, "vehave", nthreads)
    createPcf(args.output_name, args.output_dir, prvEvents)
    end = time.time()

    post_time = round(end - start, 2)
    printMsg("Processing time: " + str(post_time) + " seconds", "INFO")


if __name__ == "__main__":
    main()
